#include "Lens.hh"

#include <glm/ext.hpp>

void glow::camera::Lens::setViewportSize(int w, int h)
{
    mViewportSize.x = w;
    mViewportSize.y = h;
    mAspectRatio = static_cast<float>(w) / static_cast<float>(h);
}

glm::mat4 glow::camera::Lens::getProjectionMatrix() const
{
    if (std::isinf(mFarPlane))
    {
        float const e = 1.0f / tan(glm::radians(getVerticalFov() * 0.5f));

        // infinite Perspective matrix reversed mapping to 1..-1
        return {e / mAspectRatio, 0.0f, 0.0f, 0.0f, 0.0f, e, 0.0f, 0.0f, 0.0f, 0.0f, -1.0f, -1.0f, 0.0f, 0.0f, -2.0f * mNearPlane, 0.0f};
    }
    else
    {
        return glm::perspective(glm::radians(mHorizontalFov), mAspectRatio, mNearPlane, mFarPlane);
    }
}

float glow::camera::Lens::getVerticalFov() const
{
    return glm::degrees(atan(tan(glm::radians(0.5f * mHorizontalFov)) / mAspectRatio) * 2.0f);
}
