#pragma once

#include <glm/glm.hpp>

#include <glow/common/property.hh>

namespace glow
{
namespace camera
{
/**
 * The Lens
 *
 * Component of the camera responsible for projection
 */
class Lens
{
private:
    // == Parameters ==
    glm::ivec2 mViewportSize;
    float mAspectRatio = 1.f;

    float mHorizontalFov = 80.f;

    float mNearPlane = .1f;
    float mFarPlane = 1000.f;

public:
    // == Setters ==
    void setNearPlane(float nearPlane) { mNearPlane = nearPlane; }
    void setFarPlane(float farPlane) { mFarPlane = farPlane; }
    void setFoV(float fov) { mHorizontalFov = fov; }
    void setViewportSize(int w, int h);

    // == Getters ==
    GLOW_GETTER(ViewportSize);
    GLOW_GETTER(AspectRatio);
    GLOW_GETTER(HorizontalFov);
    GLOW_GETTER(NearPlane);
    GLOW_GETTER(FarPlane);

    glm::mat4 getProjectionMatrix() const;
    float getVerticalFov() const;
};
}
}
