#pragma once

#include <algorithm>
#include <vector>

#include <glm/vec2.hpp>

namespace glow
{
namespace input
{
struct InputSnapshot
{
    std::vector<int> pressedKeys;
    std::vector<int> pressedMouseButtons;

    glm::dvec2 mousePosition = {0, 0};
    glm::vec2 scrollPosition = {0, 0};

public:
    // -- Manipulators --
    void addKey(int key) { pressedKeys.push_back(key); }
    void removeKey(int key) { pressedKeys.erase(std::remove(pressedKeys.begin(), pressedKeys.end(), key), pressedKeys.end()); }

    void addMouseButton(int btn) { pressedMouseButtons.push_back(btn); }
    void removeMouseButton(int btn)
    {
        pressedMouseButtons.erase(std::remove(pressedMouseButtons.begin(), pressedMouseButtons.end(), btn), pressedMouseButtons.end());
    }

    void addMouseDelta(glm::dvec2 const& delta) { mousePosition += delta; }
    void addScrollDelta(glm::vec2 const& delta) { scrollPosition += delta; }

public:
    // -- Getters --
    bool isKeyDown(int key) const { return std::find(pressedKeys.begin(), pressedKeys.end(), key) != pressedKeys.end(); }
    bool isKeyUp(int key) const { return !isKeyDown(key); }
    bool isKeyPressed(InputSnapshot const& prev, int key) const { return isKeyDown(key) && prev.isKeyUp(key); }
    bool isKeyReleased(InputSnapshot const& prev, int key) const { return isKeyUp(key) && prev.isKeyDown(key); }

    bool isMouseButtonDown(int btn) const
    {
        return std::find(pressedMouseButtons.begin(), pressedMouseButtons.end(), btn) != pressedMouseButtons.end();
    }
    bool isMouseButtonUp(int btn) const { return !isMouseButtonDown(btn); }
    bool isMouseButtonPressed(InputSnapshot const& prev, int btn) const { return isMouseButtonDown(btn) && prev.isMouseButtonUp(btn); }
    bool isMouseButtonReleased(InputSnapshot const& prev, int btn) const { return isMouseButtonUp(btn) && prev.isMouseButtonDown(btn); }

    glm::dvec2 getMouseDelta(InputSnapshot const& prev) const { return mousePosition - prev.mousePosition; }
    glm::vec2 getScrollDelta(InputSnapshot const& prev) const { return scrollPosition - prev.scrollPosition; }
};
}
}
