#pragma once

#include <vector>

#include <glow/common/property.hh>
#include <glow/common/shared.hh>

#include "aabb.hh"

namespace glow
{
namespace viewer
{
GLOW_SHARED(class, Scene);
GLOW_SHARED(class, Renderable);
GLOW_SHARED(class, Light);

class Scene : public std::enable_shared_from_this<Scene>
{
    // members
private:
    std::vector<SharedRenderable> mRenderables;
    std::vector<SharedLight> mLights;
    glm::vec3 mBackgroundColor = glm::vec3(.8f);
    float mScale = 1.0f; // relative scale of the scene

    // properties
public:
    GLOW_GETTER(Renderables);
    GLOW_GETTER(Lights);
    GLOW_BUILDER(backgroundColor, BackgroundColor);
    GLOW_PROPERTY(Scale);

    // methods
public:
    /// adds a renderable
    /// returns this
    SharedScene add(SharedRenderable const& r);
    /// adds a light
    /// returns this
    SharedScene add(SharedLight const& l);

    /// computes the AABB of the scene
    aabb computeAabb() const;

    // ctor
public:
    Scene();

    static SharedScene create();
};
}
}
