#pragma once

#include <unordered_set>

#include <glow/common/property.hh>
#include <glow/common/shared.hh>
#include <glow/fwd.hh>

#include <glow-extras/camera/SmoothedCamera.hh>

#include "fwd.hh"

namespace glow
{
namespace viewer
{
class SubView : public std::enable_shared_from_this<SubView>
{
    // members
private:
    SharedScene mScene;
    SharedLayout mLayout;
    std::vector<SharedSubView> mSubViews;
    camera::SharedSmoothedCamera mCamera;

    bool mContainer = false; ///< if true, contains sub views

    // gfx
private:
    glm::ivec2 mViewportStart;
    glm::ivec2 mViewportSize;
    pipeline::SharedRenderScene mPipelineScene;
    pipeline::SharedStageCamera mPipelineCamera;

public:
    void initRendering();
    void performLayout(glm::ivec2 start, glm::ivec2 size);

    // properties
public:
    GLOW_BUILDER(scene, Scene);
    GLOW_BUILDER(camera, Camera);

    GLOW_PROPERTY(Layout);
    GLOW_PROPERTY_IS(Container);

    GLOW_GETTER(ViewportStart);
    GLOW_GETTER(ViewportSize);
    GLOW_GETTER(SubViews);

    GLOW_GETTER(PipelineScene);

    // methods
public:
    /// If no view is currently active, creates a windows and shows this view
    /// Otherwise, attaches to the current view
    void show();

    void addSubView(SharedSubView const& view);

    SharedTextureRectangle render(int w = -1, int h = -1, std::string const& fileName = "");

    // ctor
public:
    SubView();

    /// creates a new, empty 3D view
    static SharedSubView create();

    friend class ViewerApp;
};
}
}
