#pragma once

#include <unordered_set>

#include "fwd.hh"

#include <glow-extras/glfw/GlfwApp.hh>

#include "ViewRenderContext.hh"

namespace glow
{
namespace viewer
{
class ViewerApp : public glfw::GlfwApp
{
public:
    struct TerminateException : std::exception
    {
        const char* what() const noexcept { return "Global Viewer Temination"; }
    };

    // logic
private:
    SubView& mRootView;
    ViewRenderContext mContext;

    // gfx
private:
    pipeline::SharedRenderPipeline mPipeline;

    SharedUniformBuffer mUboGlobals;

    SharedTextureCubeMap mEnvironmentMap;

    mutable std::unordered_set<Program*> mRegisteredForwardShaders;

public:
    ViewerApp(SubView& view);

protected:
    void init() override;
    void update(float elapsedSeconds) override;
    void render(float elapsedSeconds) override;

    void onPerformShadowPass(pipeline::RenderContext const& ctx, UsedProgram& shader) override;
    void onRenderDepthPrePass(pipeline::RenderContext const& ctx) override;
    void onRenderOpaquePass(pipeline::RenderContext const& ctx) override;
    void onRenderTransparentPass(pipeline::RenderContext const& ctx) override;

    bool onMouseButton(double x, double y, int button, int action, int mods, int clickCount) override;
    bool onKey(int key, int scancode, int action, int mods) override;

    void onResize(int w, int h) override;
    void onGui() override;

private:
    void prepareShader(SharedProgram const& shader, UsedProgram& usedShader) const;
    void updateUniformBuffer(pipeline::RenderContext const& ctx);
    friend ViewRenderContext;
};
}
}
