#pragma once

#include <vector>

#include "fwd.hh"
#include "traits.hh"

#include <glm/mat4x4.hpp>

#include <glow-extras/colors/color.hh>

// view configuration
// is called from the variadic view(obj, ...) function

namespace glow
{
namespace viewer
{
/// overrides the transform of the given renderable
void configure(SharedSubView const& v, SharedScene const& s, Renderable& r, glm::mat4 const& transform);

/// global color of the renderable
void configure(SharedSubView const& v, SharedScene const& s, GeometricRenderable& r, glow::colors::color const& c);
template <class Color, class = std::enable_if_t<detail::is_color3_like<Color> || detail::is_color4_like<Color>>>
void configure(SharedSubView const& v, SharedScene const& s, GeometricRenderable& r, Color const& c);

/// per-primitive color of the renderable
void configure(SharedSubView const& v, SharedScene const& s, GeometricRenderable& r, polymesh::vertex_attribute<glow::colors::color> const& c);
void configure(SharedSubView const& v, SharedScene const& s, GeometricRenderable& r, polymesh::face_attribute<glow::colors::color> const& c);
void configure(SharedSubView const& v, SharedScene const& s, GeometricRenderable& r, polymesh::edge_attribute<glow::colors::color> const& c);
void configure(SharedSubView const& v, SharedScene const& s, GeometricRenderable& r, polymesh::halfedge_attribute<glow::colors::color> const& c);
void configure(SharedSubView const& v, SharedScene const& s, GeometricRenderable& r, std::vector<glow::colors::color> const& c);

/// mapping from data to color
void configure(SharedSubView const& v, SharedScene const& s, GeometricRenderable& r, ColorMapping const& cm);

/// textured mesh
void configure(SharedSubView const& v, SharedScene const& s, GeometricRenderable& r, Texturing const& t);

// ================== Implementation ==================
template <class Color, class>
void configure(SharedSubView const& v, SharedScene const& s, GeometricRenderable& r, Color const& c)
{
    // TODO: .a
    configure(v, s, r, glow::colors::color(c.r, c.g, c.b));
}
}
}
