#include "MeshDefinition.hh"

#include <glow/objects/ElementArrayBuffer.hh>

#include <polymesh/algorithms/properties.hh>

#include "MeshAttribute.hh"

using namespace glow;
using namespace glow::viewer;
using namespace glow::viewer::detail;

aabb PolyMeshDefinition::computeAABB()
{
    if (mesh.vertices().empty())
        return aabb::empty();

    auto p0 = pos[mesh.vertices().first()];

    auto bb = aabb(p0, p0);

    for (auto v : mesh.vertices())
        bb.add(pos[v]);

    return bb;
}

SharedMeshAttribute PolyMeshDefinition::computePositionAttribute()
{
    return make_mesh_attribute("aPosition", pos);
}

SharedElementArrayBuffer PolyMeshDefinition::computeIndexBuffer()
{
    if (is_triangle_mesh(mesh))
        return nullptr;

    std::vector<int> indices;
    indices.clear();
    indices.reserve(mesh.faces().size() * 3);

    auto v_base = 0;
    for (auto f : mesh.faces())
    {
        auto idx = 0;
        for (auto h : f.halfedges())
        {
            (void)h; // unused

            if (idx >= 2)
            {
                indices.push_back(v_base + 0);
                indices.push_back(v_base + idx - 1);
                indices.push_back(v_base + idx - 0);
            }

            ++idx;
        }

        v_base += idx;
    }

    return ElementArrayBuffer::create(indices);
}

SharedMeshAttribute PolyMeshDefinition::computeFaceNormalAttribute()
{
    return make_mesh_attribute("aNormal", face_normals(mesh, pos));
}

SharedMeshAttribute PolyMeshDefinition::computeSmoothNormalAttribute()
{
    return make_mesh_attribute("aNormal", vertex_normals_by_area(mesh, pos));
}
