#include "GridLayout.hh"

#include <glow/common/log.hh>

#include "../SubView.hh"

namespace glow
{
namespace viewer
{
void GridLayout::performLayout(SubView& view) const
{
    assert(view.isContainer());

    auto scnt = int(view.getSubViews().size());
    if (scnt == 0)
        return; // no sub views

    auto const vs = view.getViewportSize();
    auto rows = mRowCount;
    auto cols = mColumnCount;

    // auto-grid
    if (rows == -1 && cols == -1)
    {
        auto bestVal = std::numeric_limits<float>::max();

        for (auto ir = 1; ir <= scnt; ++ir)
        {
            auto r = ir;
            auto c = (scnt + r - 1) / r;
            while ((r - 1) * c >= scnt)
                --r;

            assert(r * c >= scnt);
            assert((r - 1) * c <= scnt);
            assert(r * (c - 1) <= scnt);
            assert(r >= 1);
            assert(c >= 1);

            auto ar = (vs.x / float(c)) / (vs.y / float(r));
            auto val = glm::max(mTargetAspectRatio / ar, ar / mTargetAspectRatio);
            if (val < bestVal)
            {
                bestVal = val;
                rows = r;
                cols = c;
            }
        }
    }
    else if (rows == -1) // fixed cols
    {
        assert(cols > 0);
        rows = (scnt + cols - 1) / cols;
    }
    else if (cols == -1) // fixed rows
    {
        assert(rows > 0);
        cols = (scnt + rows - 1) / rows;
    }

    assert(rows * cols >= scnt);
    assert(rows >= 1);
    assert(cols >= 1);

    for (auto i = 0; i < scnt; ++i)
    {
        auto c = i % cols;
        auto r = i / cols;
        auto& v = *view.getSubViews()[i];

        auto start = view.getViewportStart() + glm::ivec2(glm::floor(glm::vec2(vs) * glm::vec2(c, r) / glm::vec2(cols, rows)));
        auto end = view.getViewportStart() + glm::ivec2(glm::floor(glm::vec2(vs) * glm::vec2(c + 1, r + 1) / glm::vec2(cols, rows)));

        v.performLayout(start, end - start - 1);
    }
}
}
}
