#pragma once

#include "GeometricRenderableBuilder.hh"

namespace glow
{
namespace viewer
{
namespace builder
{
class LineBuilder : public GeometricRenderableBuilder<LineBuilder>
{
public:
    LineBuilder& round_caps()
    {
        mRoundCaps = true;
        if (mSquareCaps)
            glow::warning() << "square caps cannot be round";
        if (mNoCaps)
            glow::warning() << "no caps cannot be round";
        return *this;
    }
    LineBuilder& square_caps()
    {
        mSquareCaps = true;
        if (mRoundCaps)
            glow::warning() << "round caps cannot be square";
        if (mNoCaps)
            glow::warning() << "no caps cannot be square";
        return *this;
    }
    LineBuilder& no_caps()
    {
        mNoCaps = true;
        if (mRoundCaps)
            glow::warning() << "round caps cannot be none";
        if (mSquareCaps)
            glow::warning() << "square caps cannot be none";
        if (mExtrapolate)
            glow::warning() << "extrapolated caps cannot be none";
        return *this;
    }
    LineBuilder& camera_facing()
    {
        mCameraFacing = true;
        if (mOwnNormals)
            glow::warning() << "points with normals cannot be camera facing";
        return *this;
    }
    LineBuilder& extrapolate()
    {
        mExtrapolate = true;
        if (mNoCaps)
            glow::warning() << "no caps cannot be extrapolated";
        return *this;
    }

    template <class T>
    LineBuilder& normals(T const& val)
    {
        addAttribute(detail::make_mesh_attribute("aNormal", val));
        mOwnNormals = true;
        if (mScreenSpaceSize)
            glow::warning() << "screen space sized lines cannot have normals";
        if (mCameraFacing)
            glow::warning() << "camera facing lines cannot have normals";
        return *this;
    }
    template <class T>
    LineBuilder& line_width_world(T const& val)
    {
        addAttribute(detail::make_mesh_attribute("aLineWidth", val));
        mWorldSpaceSize = true;
        if (mScreenSpaceSize)
            glow::warning() << "lines with screen space width cannot have a world space width";
        return *this;
    }
    template <class T>
    LineBuilder& line_width_px(T const& val)
    {
        addAttribute(detail::make_mesh_attribute("aLineWidth", val));
        mScreenSpaceSize = true;
        if (mOwnNormals)
            glow::warning() << "screen space lines cannot have normals";
        if (mWorldSpaceSize)
            glow::warning() << "lines with world space width cannot have a screen space width";
        return *this;
    }

private:
    bool mCameraFacing = false;
    bool mOwnNormals = false;
    bool mWorldSpaceSize = false;
    bool mScreenSpaceSize = false;
    bool mSquareCaps = false;
    bool mRoundCaps = false;
    bool mNoCaps = false;
    bool mExtrapolate = false;

    std::shared_ptr<detail::PolyMeshDefinition> mMeshDef;

public:
    GLOW_GETTER(MeshDef);

public:
    explicit LineBuilder(std::shared_ptr<detail::PolyMeshDefinition> def) : mMeshDef(std::move(def)) {}
    friend class glow::viewer::LineRenderable;
};
}
}
}
