#pragma once

#include "GeometricRenderable.hh"

#include "../objects/geometry/PointBuilder.hh"

namespace glow
{
namespace viewer
{
class PointRenderable final : public GeometricRenderable
{
private:
    enum class RenderMode
    {
        Round,
        Square,
        Sphere
    };

    RenderMode mRenderMode = RenderMode::Round;
    bool mCameraFacing = false;
    bool mWorldSpaceSize = false;

    // is lazily built
    glow::SharedVertexArray mVertexArray;
    glow::SharedProgram mForwardShader;

public:
    aabb computeAabb() override;

    void renderShadow(ViewRenderContext const& ctx, glow::UsedProgram& shader) override;
    void renderDepthPre(ViewRenderContext const& ctx) override;
    void renderForward(ViewRenderContext const& ctx) override;
    void renderTransparent(ViewRenderContext const& ctx) override;

public:
    static SharedPointRenderable create(builder::PointBuilder const& builder);

private:
    void initFromBuilder(builder::PointBuilder const& builder);

    void init() override;
};
}
}
