#include <glow-pipeline/pass/opaque/opaquePass.glsl>
#include <glow-material/material-ggx.glsl>

layout (std140) uniform uViewerGlobalsUBO
{
    mat4 view;
    mat4 proj;
    mat4 invView;
    mat4 invProj;
    mat4 cleanVp;
    mat4 prevCleanVp;

    vec3 camPos;
    vec3 sunDirection;
    vec3 sunColor;
} uViewerGlobals;

// Performs no shading or processing
void outputViewerGeometryRaw(vec3 color)
{
    outputOpaqueGeometry(color, getFragmentVelocity(vIn.HDCPosition, vIn.prevHDCPosition));
}

// Outputs geometry that is shaded and lit in the default viewer flavour
void outputViewerGeometry(vec3 albedo, vec3 N)
{
    if (N == vec3(0,0,0))
    {
        outputViewerGeometryRaw(albedo);
        return;
    }

    N = normalize(N);

    const float roughness = 0.9;
    const float metallic = 0.35;

    vec3 cameraDistanceVector = uViewerGlobals.camPos - vIn.fragPosWS;
    vec3 V = normalize(cameraDistanceVector);
    float cameraDistanceSquared = dot(cameraDistanceVector, cameraDistanceVector);

    vec3 color = vec3(0, 0, 0);

    // IBL + AO
    color += applyAo(4.0 * iblSpecularGGX(N, V, albedo, roughness, metallic));

    // Sunlight
    color += getShadowVisibility(vIn.fragPosWS) * uViewerGlobals.sunColor * shadingGGX(N, V, uViewerGlobals.sunDirection, albedo, roughness, metallic);

    // Tube Lights
    FOREACH_LIGHT(i)
    {
        LightData light = getLight(i);

        vec3 L;
        vec3 radiance;
        getTubeLightInfo(light, vIn.fragPosWS, V, N, roughness, L, radiance);

        color += radiance * shadingGGX(N, V, L, albedo, roughness, metallic);
    }

    // Pipeline Output
    outputOpaqueGeometry(color, getFragmentVelocity(vIn.HDCPosition, vIn.prevHDCPosition));
}
