layout (std140) uniform uViewerGlobalsUBO
{
    mat4 view;
    mat4 proj;
    mat4 invView;
    mat4 invProj;
    mat4 cleanVp;
    mat4 prevCleanVp;

    vec3 camPos;
    vec3 sunDirection;
    vec3 sunColor;
} uViewerGlobals;

// Calculates and outputs values required for internal fragment helpers
void outputViewerVertexInternal(vec3 pos, mat4 modelMatrix)
{
    vec4 worldSpace = modelMatrix * vec4(pos, 1.0);
    vOut.fragPosWS = vec3(worldSpace);

    vOut.HDCPosition = uViewerGlobals.cleanVp * modelMatrix * vec4(pos, 1.0);
    vOut.prevHDCPosition = uViewerGlobals.prevCleanVp * modelMatrix * vec4(pos, 1.0);
}

// Calculates and outputs values required for internal fragment helpers, and sets gl_Position
void outputViewerVertex(vec3 pos, mat4 modelMatrix)
{
    outputViewerVertexInternal(pos, modelMatrix);
    gl_Position = uViewerGlobals.proj * uViewerGlobals.view * modelMatrix * vec4(pos, 1.0);
}

void outputViewerVertexInverse(vec4 pos)
{
    gl_Position = pos;
    pos = uViewerGlobals.invProj * pos;
    pos /= pos.w;
    pos = uViewerGlobals.invView * pos;
    vOut.fragPosWS = vec3(pos);

    vOut.HDCPosition = uViewerGlobals.cleanVp * pos;
    vOut.prevHDCPosition = uViewerGlobals.prevCleanVp * pos;
}
